@@ -83,6 +83,7 @@ static const NSString *kDisableSharingCalendar = @"Calendar";
 - (BOOL) uixDebugEnabled;
 - (BOOL) easDebugEnabled;
 - (BOOL) openIdDebugEnabled;
+- (BOOL) apiDebugEnabled;
 - (BOOL) tnefDecoderDebugEnabled;
 - (BOOL) xsrfValidationEnabled;
 
@@ -106,6 +107,7 @@ NSComparisonResult languageSort(id el1, id el2, void *context);
 - (NSString *) openIdClient;
 - (NSString *) openIdClientSecret;
 - (NSString *) openIdEmailParam;
+- (NSString *) openIdHttpVersion;
 - (BOOL) openIdEnableRefreshToken;
 - (BOOL) openIdLogoutEnabled: (NSString *) _domain;
 - (int) openIdTokenCheckInterval;
