@@ -654,6 +654,11 @@ _injectConfigurationFromFile (NSMutableDictionary *defaultsDict,
   return [self boolForKey: @"SOGoOpenIDDebugEnabled"];
 }
 
+- (BOOL) apiDebugEnabled
+{
+  return [self boolForKey: @"SOGoAPIDebugEnabled"];
+}
+
 - (BOOL) tnefDecoderDebugEnabled
 {
   return [self boolForKey: @"SOGoTnefDecoderDebugEnabled"];
@@ -779,6 +784,15 @@ NSComparisonResult languageSort(id el1, id el2, void *context)
   return emailParam;
 }
 
+- (NSString *) openIdHttpVersion
+{
+  NSString *httpVersion;
+  httpVersion = [self stringForKey: @"SOGoOpenIdHttpVersion"];
+  if(!httpVersion)
+    httpVersion = @"HTTP/1.1";
+  return httpVersion;
+}
+
 - (BOOL) openIdLogoutEnabled: (NSString *) _domain
 {
   if(_domain && [self doesLoginTypeByDomain])
