@@ -1,6 +1,6 @@
 /* UIxMailActions.h - this file is part of SOGo
  *
- * Copyright (C) 2007-2013 Inverse inc.
+ * Copyright (C) 2007-2016 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,9 +21,9 @@
 #ifndef UIXMAILACTIONS_H
 #define UIXMAILACTIONS_H
 
-#import <NGObjWeb/WODirectAction.h>
+#import <UI/SOGoUI/SOGoDirectAction.h>
 
-@interface UIxMailActions : WODirectAction
+@interface UIxMailActions : SOGoDirectAction
 @end
 
 #endif /* UIXMAILACTIONS_H */
