@@ -40,6 +40,7 @@
 #import <SOGo/SOGoBuild.h>
 #import <SOGo/SOGoCache.h>
 #import <SOGo/SOGoCASSession.h>
+#import <SOGo/SOGoOpenIdSession.h>
 #if defined(SAML2_CONFIG)
 #import <SOGo/SOGoSAML2Session.h>
 #endif /* SAML2_ENABLE */
@@ -52,6 +53,7 @@
 #import <SOGo/SOGoEmptyAuthenticator.h>
 #import <SOGo/SOGoMailer.h>
 #import <SOGo/SOGoAdmin.h>
+#import <SOGo/SOGoPasswordPolicy.h>
 
 #if defined(MFA_CONFIG)
 #include <liboath/oath.h>
@@ -144,6 +146,7 @@ static const NSString *kJwtKey = @"jwt";
 
 - (WOCookie *) _authLocationCookie: (BOOL) cookieReset
                           withName: (NSString *) cookieName
+                          withValue: (NSString *) _value
 {
   WOCookie *locationCookie;
   NSString *appName;
@@ -151,7 +154,10 @@ static const NSString *kJwtKey = @"jwt";
   NSCalendarDate *date;
 
   rq = [context request];
-  locationCookie = [WOCookie cookieWithName: cookieName value: [rq uri]];
+  if(_value)
+    locationCookie = [WOCookie cookieWithName: cookieName value: _value];
+  else
+    locationCookie = [WOCookie cookieWithName: cookieName value: [rq uri]];
   appName = [rq applicationName];
   [locationCookie setPath: [NSString stringWithFormat: @"/%@/", appName]];
   if (cookieReset)
@@ -164,17 +170,48 @@ static const NSString *kJwtKey = @"jwt";
   return locationCookie;
 }
 
+- (WOCookie *) _domainCookie: (BOOL) cookieReset
+                    withDomain: (NSString *) _domain
+{
+  WOCookie *domainCookie;
+  NSString *appName;
+  WORequest *rq;
+  NSCalendarDate *date;
+
+  rq = [context request];
+  domainCookie = [WOCookie cookieWithName: @"sogo-user-domain" value: _domain];
+  appName = [rq applicationName];
+  [domainCookie setPath: [NSString stringWithFormat: @"/%@/", appName]];
+  if (cookieReset)
+    {
+      date = [NSCalendarDate calendarDate];
+      [date setTimeZone: [NSTimeZone timeZoneForSecondsFromGMT: 0]];
+      [domainCookie setExpires: [date yesterday]];
+    }
+
+  return domainCookie;
+}
+
 //
 //
 //
-- (WOResponse *) _responseWithLDAPPolicyError: (int) error
+- (WOResponse *) _responseWithLDAPPolicyError: (int) error additionalInfos: (NSDictionary *) additionalInfos
 {
   NSDictionary *jsonError;
 
-  jsonError = [NSDictionary dictionaryWithObject: [NSNumber numberWithInt: error]
-                                          forKey: @"LDAPPasswordPolicyError"];
-  return [self responseWithStatus: 403
-            andJSONRepresentation: jsonError];
+  if (additionalInfos) {
+    jsonError = [NSDictionary dictionaryWithObjectsAndKeys:
+                 [NSNumber numberWithInt:error], @"LDAPPasswordPolicyError",
+                 additionalInfos, @"additionalInfos",
+                 nil];
+  } else {
+    jsonError = [NSDictionary dictionaryWithObjectsAndKeys:
+                 [NSNumber numberWithInt:error], @"LDAPPasswordPolicyError",
+                 nil];
+  }
+
+  return [self responseWithStatus:403
+            andJSONRepresentation:jsonError];
 }
 
 - (void) _checkAutoReloadWebCalendars: (SOGoUser *) loggedInUser
@@ -197,6 +234,27 @@ static const NSString *kJwtKey = @"jwt";
     }
 }
 
+//
+//
+//
+- (void)translateAdditionalLoginInformations:(NSMutableDictionary **)additionalLoginInformations
+{
+  NSDictionary *policy;
+  NSMutableDictionary *translations;
+
+  if (additionalLoginInformations && *additionalLoginInformations) {
+    if ([*additionalLoginInformations objectForKey:@"userPolicies"]) {
+      translations = [[NSMutableDictionary alloc] init];
+      for (policy in [*additionalLoginInformations objectForKey:@"userPolicies"]) {
+        [translations setObject:[self commonLabelForKey: [policy objectForKey:@"label"]] forKey: [policy objectForKey:@"label"]];
+      }
+      [*additionalLoginInformations setObject:[SOGoPasswordPolicy createPasswordPolicyLabels: [*additionalLoginInformations objectForKey:@"userPolicies"] withTranslations: translations] 
+                    forKey:@"userPolicies"];
+      [translations release];
+    }
+  }
+}
+
 //
 //
 //
@@ -210,6 +268,7 @@ static const NSString *kJwtKey = @"jwt";
   SOGoUserSettings *us;
   SOGoUser *loggedInUser;
   NSDictionary *params;
+  NSMutableDictionary *additionalLoginInformations;
   NSString *username, *password, *language, *domain, *remoteHost;
   NSArray *supportedLanguages, *creds;
 
@@ -223,6 +282,7 @@ static const NSString *kJwtKey = @"jwt";
   auth = [[WOApplication application] authenticatorInContext: context];
   request = [context request];
   params = [[request contentAsString] objectFromJSONString];
+  additionalLoginInformations = [[NSMutableDictionary alloc] init];
 
   username = [params objectForKey: @"userName"];
   password = [params objectForKey: @"password"];
@@ -232,9 +292,11 @@ static const NSString *kJwtKey = @"jwt";
   /* this will always be set to something more or less useful by
    * [WOHttpTransaction applyAdaptorHeadersWithHttpRequest] */
   remoteHost = [request headerForKey:@"x-webobjects-remote-host"];
+  b = [auth checkLogin: username password: password domain: &domain
+		 perr: &err expire: &expire grace: &grace additionalInfo: &additionalLoginInformations useCache: NO];
+  [self translateAdditionalLoginInformations: &additionalLoginInformations];
 
-  if ((b = [auth checkLogin: username password: password domain: &domain
-		 perr: &err expire: &expire grace: &grace useCache: NO])
+  if (b
       && (err == PolicyNoError)
       // no password policy
       && ((expire < 0 && grace < 0)     // no password policy or everything is alright
@@ -334,7 +396,7 @@ static const NSString *kJwtKey = @"jwt";
 #endif
       
       if ([us objectForKey: @"ForceResetPassword"]) {
-        response = [self _responseWithLDAPPolicyError: PolicyPasswordExpired];
+        response = [self _responseWithLDAPPolicyError: PolicyPasswordExpired additionalInfos: additionalLoginInformations];
       } else {
         [self _checkAutoReloadWebCalendars: loggedInUser];
 
@@ -377,7 +439,7 @@ static const NSString *kJwtKey = @"jwt";
       [self logWithFormat: @"Login from '%@' for user '%@' might not have worked - password policy: %d  grace: %d  expire: %d  bound: %d",
             remoteHost, username, err, grace, expire, b];
 
-      response = [self _responseWithLDAPPolicyError: err];
+      response = [self _responseWithLDAPPolicyError: err additionalInfos: additionalLoginInformations];
     }
 
   if (rememberLogin)
@@ -385,9 +447,12 @@ static const NSString *kJwtKey = @"jwt";
   else
     [response addCookie: [self _cookieWithUsername: nil]];
 
+  [additionalLoginInformations release];
+
   return response;
 }
 
+
 - (NSDictionary *) _casRedirectKeys
 {
   NSDictionary *redirectKeys;
@@ -441,42 +506,43 @@ static const NSString *kJwtKey = @"jwt";
   if ([login isEqualToString: @"anonymous"])
     login = nil;
   if (!login)
+  {
+    rq = [context request];
+    ticket = [rq formValueForKey: @"ticket"];
+    if ([ticket length])
     {
-      rq = [context request];
-      ticket = [rq formValueForKey: @"ticket"];
-      if ([ticket length])
-        {
-          casSession = [SOGoCASSession CASSessionWithTicket: ticket
-                                                  fromProxy: NO];
-          login = [casSession login];
-          if ([login length])
-            {
-              auth = [[WOApplication application]
-                       authenticatorInContext: context];
-              casCookie = [auth cookieWithUsername: login
-                                       andPassword: [casSession identifier]
-                                         inContext: context];
-              [casSession updateCache];
-              newLocation = [rq cookieValueForKey: @"cas-location"];
-              /* login callback, we expire the "cas-location" cookie, created
-                 below */
-              casLocationCookie = [self _authLocationCookie: YES
-                                                   withName: @"cas-location"];
-            }
-        }
-      else
-        {
-          /* anonymous and no ticket, possibly a logout request from CAS
-           * See: https://wiki.jasig.org/display/CASUM/Single+Sign+Out
-           */
-          logoutRequest = [rq formValueForKey: @"logoutRequest"];
-          if ([logoutRequest length])
-            {
-              [SOGoCASSession handleLogoutRequest: logoutRequest];
-              return [self responseWithStatus: 200];
-            }
-        }
+      casSession = [SOGoCASSession CASSessionWithTicket: ticket
+                                              fromProxy: NO];
+      login = [casSession login];
+      if ([login length])
+      {
+        auth = [[WOApplication application]
+                  authenticatorInContext: context];
+        casCookie = [auth cookieWithUsername: login
+                                  andPassword: [casSession identifier]
+                                    inContext: context];
+        [casSession updateCache];
+        newLocation = [rq cookieValueForKey: @"cas-location"];
+        /* login callback, we expire the "cas-location" cookie, created
+            below */
+        casLocationCookie = [self _authLocationCookie: YES
+                                              withName: @"cas-location"
+                                              withValue: nil];
+      }
     }
+    else
+      {
+        /* anonymous and no ticket, possibly a logout request from CAS
+          * See: https://wiki.jasig.org/display/CASUM/Single+Sign+Out
+          */
+        logoutRequest = [rq formValueForKey: @"logoutRequest"];
+        if ([logoutRequest length])
+          {
+            [SOGoCASSession handleLogoutRequest: logoutRequest];
+            return [self responseWithStatus: 200];
+          }
+      }
+  }
   else
     ticket = nil;
 
@@ -500,7 +566,8 @@ static const NSString *kJwtKey = @"jwt";
       newLocation = [SOGoCASSession CASURLWithAction: @"login"
                                        andParameters: [self _casRedirectKeys]];
       casLocationCookie = [self _authLocationCookie: NO
-                                           withName: @"cas-location"];
+                                           withName: @"cas-location"
+                                           withValue: nil];
     }
   response = [self redirectToLocation: newLocation];
   if (casCookie)
@@ -511,6 +578,131 @@ static const NSString *kJwtKey = @"jwt";
   return response;
 }
 
+- (id <WOActionResults>) _openidDefaultAction: (NSString *) _domain
+{
+  WOResponse *response;
+  NSString *login, *redirectLocation, *serverUrl;
+  NSString *sessionState, *code, *refreshTokenValue;
+  NSURL *newLocation, *oldLocation;
+  NSDictionary *formValues;
+  SOGoUser *loggedInUser;
+  WOCookie *openIdCookie, *openIdCookieLocation, *openIdRefreshCookie, *domainCookie;
+  WORequest *rq;
+  SOGoWebAuthenticator *auth;
+  SOGoOpenIdSession *openIdSession;
+  id value;
+
+  openIdCookie = nil;
+  openIdCookieLocation = nil;
+  openIdRefreshCookie = nil;
+  domainCookie = nil;
+  newLocation = nil;
+
+  rq = [context request];
+
+  //Check if the domain is stored in a cookie if not given
+  if(_domain == nil || [_domain length] == 0)
+    _domain = [rq cookieValueForKey: @"sogo-user-domain"]; //_domain can still be nil aftert his
+
+  openIdSession = [SOGoOpenIdSession OpenIdSession: _domain];
+
+  if(![openIdSession sessionIsOk])
+  {
+    return [NSException exceptionWithHTTPStatus: 502 /* Bad OpenId Configuration */
+                                       reason: [NSString stringWithFormat: @"OpenId server not found or has unexpected behavior, contact your admin."]];
+  }
+
+  login = [[context activeUser] login];
+  if ([login isEqualToString: @"anonymous"])
+    login = nil;
+  if (!login)
+  {
+    //You get here if you nerver been logged in or if you token is expired
+    serverUrl = [[context serverURL] absoluteString];
+    redirectLocation = [NSString stringWithFormat: @"%@/%@/", serverUrl, [rq applicationName]];
+    if((formValues = [rq formValues]) && [formValues objectForKey: @"code"])
+    {
+      //You get here if this is the callback of openid after you logged in
+
+      //NOT MANDATORY
+      // value = [formValues objectForKey: @"session_state"];
+      // if ([value isKindOfClass: [NSArray class]])
+      //   sessionState = [value lastObject];
+      // else
+      //   sessionState = value;
+
+      value = [formValues objectForKey: @"code"];
+      if ([value isKindOfClass: [NSArray class]])
+        code = [value lastObject];
+      else
+        code = value;
+      [openIdSession fetchToken: code redirect: redirectLocation];
+      login = [openIdSession login: @""];
+      if ([login length])
+      {
+        auth = [[WOApplication application] authenticatorInContext: context];
+        openIdCookie = [auth cookieWithUsername: login
+                                  andPassword: [openIdSession getToken]
+                                    inContext: context];
+      }
+      newLocation = [rq cookieValueForKey: @"openid-location"];
+      openIdCookieLocation = [self _authLocationCookie: YES withName: @"openid-location" withValue: nil];
+      domainCookie = [self _domainCookie: YES withDomain: _domain];
+    }
+    // else if((formValues = [rq formValues]) && [formValues objectForKey: @"action"])
+    // {
+    //   value = [formValues objectForKey: @"action"];
+    //   if ([value isKindOfClass: [NSArray class]])
+    //     code = [value lastObject];
+    //   else
+    //     code = value;
+    //   if([code isEqualToString:@"redirect"])
+    //   {
+    //     //this action onlye serve to make a redirection to openId server after a GET request
+    //     newLocation = [openIdSession loginUrl: redirectLocation];
+    //   }
+    //}
+    else
+    {
+      // //You get here the first time you access sogo, it redirect to last location
+      // if([[rq method] isEqualToString: @"POST"])
+      //   //To avoid making a redirection to openid server after a post request, we first redirect to a get method
+      //   newLocation = [NSString stringWithFormat: @"%@?action=redirect", redirectLocation];
+      // else
+      if(_domain != nil && [_domain length] > 0)
+      {
+        //add the domain cookie to get it after the redirect
+        domainCookie = [self _domainCookie: NO withDomain: _domain];
+      }
+      newLocation = [openIdSession loginUrl: redirectLocation];
+      openIdCookieLocation = [self _authLocationCookie: NO withName: @"openid-location" withValue: nil];
+    }
+  }
+  else
+  {
+    if (!newLocation)
+    {
+      oldLocation = [[self clientObject] baseURLInContext: context];
+      newLocation = [NSString stringWithFormat: @"%@%@", oldLocation, [login stringByEscapingURL]];
+    }
+
+    loggedInUser = [SOGoUser userWithLogin: login];
+    [self _checkAutoReloadWebCalendars: loggedInUser];
+  }
+
+  response = [self redirectToLocation: newLocation];
+  if (openIdCookie)
+    [response addCookie: openIdCookie];
+  if (openIdCookieLocation)
+    [response addCookie: openIdCookieLocation];
+  if(domainCookie)
+    [response addCookie: domainCookie];
+  //[response setStatus: 303];
+  return response;
+}
+
+
+
 #if defined(SAML2_CONFIG)
 - (id <WOActionResults>) _saml2DefaultAction
 {
@@ -534,7 +726,8 @@ static const NSString *kJwtKey = @"jwt";
       newLocation = [rq cookieValueForKey: @"saml2-location"];
       if (newLocation)
         saml2LocationCookie = [self _authLocationCookie: YES
-                                               withName: @"saml2-location"];
+                                               withName: @"saml2-location"
+                                               withValue: nil];
       else
         {
           oldLocation = [[self clientObject] baseURLInContext: context];
@@ -549,7 +742,8 @@ static const NSString *kJwtKey = @"jwt";
     {
       newLocation = [SOGoSAML2Session authenticationURLInContext: context];
       saml2LocationCookie = [self _authLocationCookie: NO
-                                             withName: @"saml2-location"];
+                                             withName: @"saml2-location"
+                                             withValue: nil];
     }
 
   response = [self redirectToLocation: newLocation];
@@ -570,43 +764,191 @@ static const NSString *kJwtKey = @"jwt";
     login = nil;
 
   if (login)
-    {
-      oldLocation = [[self clientObject] baseURLInContext: context];
-      response
-	= [self redirectToLocation: [NSString stringWithFormat: @"%@%@",
-					      oldLocation,
-                                              [[SOGoUser getEncryptedUsernameIfNeeded:login request: [context request]] stringByEscapingURL]]];
-    }
+  {
+    oldLocation = [[self clientObject] baseURLInContext: context];
+    response = [self redirectToLocation: [NSString stringWithFormat: @"%@%@",  oldLocation,
+                  [[SOGoUser getEncryptedUsernameIfNeeded:login request: [context request]] stringByEscapingURL]]];
+  }
   else
-    {
-      oldLocation = [[context request] uri];
-      if ([context clientObject]
-          && ![oldLocation hasSuffix: @"/"]
-          && ![oldLocation hasSuffix: @"/view"])
-        response = [self redirectToLocation:
-                           [NSString stringWithFormat: @"%@/", oldLocation]];
-      else
-        response = self;
-    }
+  {
+    oldLocation = [[context request] uri];
+    if ([context clientObject] && ![oldLocation hasSuffix: @"/"] && ![oldLocation hasSuffix: @"/view"])
+      response = [self redirectToLocation: [NSString stringWithFormat: @"%@/", oldLocation]];
+    else
+      response = self;
+  }
 
   return response;
 }
 
+- (WOResponse *) connectNameAction
+{
+  WOResponse *response;
+  WORequest *request;
+  NSDictionary *params;
+  NSString *username, *language, *domain, *type, *serverUrl, *redirectLocation;
+  NSRange r;
+
+  request = [context request];
+  params = [[request contentAsString] objectFromJSONString];
+
+  username = [params objectForKey: @"userName"];
+
+  //Extract the domain
+  r = [username rangeOfString: @"@"];
+  if (r.location != NSNotFound)
+  {
+    domain = [username substringFromIndex: r.location+1];
+    type = [[SOGoSystemDefaults sharedSystemDefaults] getLoginTypeForDomain: domain];
+    if(type != nil)
+    {
+      if([type isEqualToString: @"plain"])
+      {
+        //Only reload the page with the name
+        serverUrl = [[context serverURL] absoluteString];
+        redirectLocation = [NSString stringWithFormat: @"%@/%@/login?hint=%@", serverUrl, [request applicationName], username];
+        //response = [self redirectToLocation: [NSString stringWithFormat: @"%@/", redirectLocation]];
+        response = [self responseWithStatus: 200 andJSONRepresentation:
+          [NSDictionary dictionaryWithObjectsAndKeys: redirectLocation, @"redirect", nil]];
+      }
+      else if([type isEqualToString: @"openid"])
+      {
+        SOGoOpenIdSession *openIdSession;
+        WOCookie *domainCookie, *openIdCookieLocation;
+
+        //With openId, the user will be redirected to the openid server for login
+        //With set the domain in a cookie to know it after the openid does the callbacl
+        serverUrl = [[context serverURL] absoluteString];
+        redirectLocation = [NSString stringWithFormat: @"%@/%@/", serverUrl, [request applicationName]];
+
+        openIdSession = [SOGoOpenIdSession OpenIdSession: domain];
+
+        domainCookie = [self _domainCookie: NO withDomain: domain];
+        openIdCookieLocation = [self _authLocationCookie: NO withName: @"openid-location" withValue: redirectLocation];
+
+        response = [self responseWithStatus: 200 andJSONRepresentation:
+          [NSDictionary dictionaryWithObjectsAndKeys: [openIdSession loginUrl: redirectLocation], @"redirect", nil]];
+        [response addCookie: domainCookie];
+        [response addCookie: openIdCookieLocation];
+      }
+      else if([type isEqualToString: @"cas"] || [type isEqualToString: @"saml2"])
+      {
+        [self logWithFormat: @"Unsupported type for now: %@", type];
+        response = [self responseWithStatus: 400
+                              andString: @"Domain Authentication type not supported"];
+      }
+      else
+      {
+        [self logWithFormat: @"Unknown type: %@", type];
+        response = [self responseWithStatus: 400
+                              andString: @"Unknwon Authentication type"];
+      }
+    }
+    else
+    {
+      [self logWithFormat: @"Auth type for Domain given is not set or there is no default value: %@", domain];
+      response = [self responseWithStatus: 400
+                              andString: @"Domain unknown"];
+    }
+
+  }
+  else
+  {
+    [self logWithFormat: @"Domain is required but not found for user recovery exception for user %@", username];
+    response = [self responseWithStatus: 400
+                              andString: @"Domain needed in the login"];
+  }
+
+  return response;
+}
+
+
 - (id <WOActionResults>) defaultAction
 {
-  NSString *authenticationType;
+  NSString *authenticationType, *loginDomain, *type, *_domain;
+  SOGoSystemDefaults* sd;
   id <WOActionResults> result;
 
-  authenticationType = [[SOGoSystemDefaults sharedSystemDefaults]
-                         authenticationType];
-  if ([authenticationType isEqualToString: @"cas"])
-    result = [self _casDefaultAction];
-#if defined(SAML2_CONFIG)
-  else if ([authenticationType isEqualToString: @"saml2"])
-    result = [self _saml2DefaultAction];
-#endif /* SAML2_CONFIG */
-  else
-    result = [self _standardDefaultAction];
+  loginDomain = nil;
+  sd = [SOGoSystemDefaults sharedSystemDefaults];
+  if([sd doesLoginTypeByDomain])
+  {
+    NSString *login;
+    //In this mode sogo will ask the mail of the user before doing any authentication
+    //Check if a user is already logged in
+
+    _domain = [[context request] cookieValueForKey: @"sogo-user-domain"]; //_domain can still be nil aftert his
+    if(_domain != nil)
+    {
+      //This is a callback of an openid session.
+      return [self _openidDefaultAction: _domain];
+    }
+
+    login = [[context activeUser] login];
+    if ([login isEqualToString: @"anonymous"])
+      login = nil;
+    if(!login && !_domain)
+      return [self _standardDefaultAction];
+    else
+    {
+      //User already logged in. Extract the domain in that case
+      NSRange r;
+      r = [login rangeOfString: @"@"];
+      if (r.location != NSNotFound)
+      {
+        loginDomain = [login substringFromIndex: r.location+1];
+        type = [sd getLoginTypeForDomain: loginDomain];
+        if(type)
+        {
+          if([type isEqualToString: @"plain"])
+          {
+            result = [self _standardDefaultAction];
+          }
+          else if([type isEqualToString: @"openid"])
+          {
+            result = [self _openidDefaultAction: loginDomain];
+          }
+          else if([type isEqualToString: @"cas"] || [type isEqualToString: @"saml2"])
+          {
+            [self logWithFormat: @"Unsupported type for now: %@", type];
+            result = [self responseWithStatus: 400
+                                  andString: @"Domain Authentication type not supported"];
+          }
+          else
+          {
+            [self logWithFormat: @"Unknown type: %@", type];
+            result = [self responseWithStatus: 400
+                                  andString: @"Unknwon Authentication type"];
+          }
+        }
+        else
+        {
+          [self logWithFormat: @"Auth type for Domain given is not set or there is no default value: %@", loginDomain];
+          result = [self responseWithStatus: 400
+                                  andString: @"Domain unknown"];
+        }
+      }
+      else
+      {
+        loginDomain = nil;
+        result = [self _standardDefaultAction];
+      }
+    }
+  }
+  else {
+    authenticationType = [sd authenticationType];
+
+    if ([authenticationType isEqualToString: @"cas"])
+      result = [self _casDefaultAction];
+    else if ([authenticationType isEqualToString: @"openid"])
+      result = [self _openidDefaultAction: loginDomain];
+    #if defined(SAML2_CONFIG)
+    else if ([authenticationType isEqualToString: @"saml2"])
+      result = [self _saml2DefaultAction];
+    #endif /* SAML2_CONFIG */
+    else
+      result = [self _standardDefaultAction];
+  }
 
   return result;
 }
@@ -636,6 +978,57 @@ static const NSString *kJwtKey = @"jwt";
   return ([[self loginDomains] count] > 0);
 }
 
+- (BOOL) doLoginUsernameFirst
+{
+  return [[SOGoSystemDefaults sharedSystemDefaults] doesLoginTypeByDomain];
+}
+
+- (BOOL) doFullLogin
+{
+  //Either we directly do the full login (meaning the user inputs its username and password)
+  //Or we do it in two times:
+  //phase 1: user types its username first -> only show the username input
+  //phase 2: user types its password -> show all inputs
+  //In phase 2, the username will be in the query at key "login"
+  if([self doLoginUsernameFirst]){
+    WORequest *rq;
+    BOOL hasLogin;
+    NSDictionary *formValues;
+
+    rq = [context request];
+    hasLogin = ((formValues=[rq formValues]) && [formValues objectForKey: @"hint"]);
+    return hasLogin;
+  }
+
+  return YES;
+}
+
+- (BOOL) doPartialLogin
+{
+  return ![self doFullLogin];
+}
+
+
+- (NSString *) getLoginHint
+{
+  id value;
+  WORequest *rq;
+  NSString* login;
+  NSDictionary *formValues;
+
+  login = @"";
+
+  rq = [context request];
+  if((formValues=[rq formValues]) && (value=[formValues objectForKey: @"hint"]))
+  {
+    if ([value isKindOfClass: [NSArray class]])
+      login = [value lastObject];
+    else
+      login = value;
+  }
+  return login;
+}
+
 - (BOOL) hasPasswordRecovery
 {
   return [[SOGoSystemDefaults sharedSystemDefaults] isPasswordRecoveryEnabled];
@@ -808,7 +1201,7 @@ static const NSString *kJwtKey = @"jwt";
           }
         }
       else
-        response = [self _responseWithLDAPPolicyError: error];
+        response = [self _responseWithLDAPPolicyError: error additionalInfos: nil];
     }
 
   return response;
@@ -999,8 +1392,7 @@ static const NSString *kJwtKey = @"jwt";
   message = [[request contentAsString] objectFromJSONString];
   username = [message objectForKey: @"userName"];
   domain = [message objectForKey: @"domain"];
-  if ([[SOGoSystemDefaults sharedSystemDefaults]
-                             isPasswordRecoveryEnabled]) {
+  if ([[SOGoSystemDefaults sharedSystemDefaults] isPasswordRecoveryEnabled]) {
       // If no domain, try to retrieve domain from username
       if (nil != domain && domain != [NSNull null]) {
         domainName = domain;
