@@ -26,6 +26,7 @@
 
 #import <SOGo/NSDictionary+Utilities.h>
 #import <SOGo/NSString+Utilities.h>
+#import <SOGo/SOGoSystemDefaults.h>
 
 #import <Appointments/SOGoWebAppointmentFolder.h>
 
@@ -149,4 +150,9 @@
   return response;
 }
 
+- (BOOL) isEasUIEnabled
+{
+  return ![[SOGoSystemDefaults sharedSystemDefaults] isEasUIDisabled];
+}
+
 @end
