@@ -150,6 +150,13 @@
           data.Forward.forwardAddress = [];
       }
 
+      if (data.Notification) {
+        if (angular.isString(data.Notification.notificationAddress))
+          data.Notification.notificationAddress = data.Notification.notificationAddress.split(/, */);
+        else if (!angular.isArray(data.Notification.notificationAddress))
+          data.Notification.notificationAddress = [];
+      }
+
       // Split calendar categories colors keys and values
       if (angular.isUndefined(data.SOGoCalendarCategories))
         data.SOGoCalendarCategories = [];
@@ -580,7 +587,7 @@
       }
     }).finally(function () {
       var refreshViewCheck = _this.defaults.SOGoRefreshViewCheck;
-      if (refreshViewCheck && refreshViewCheck != 'manually')
+      if (refreshViewCheck && refreshViewCheck != 'manually' && !isNaN(refreshViewCheck))
         _this.nextInboxPoll = Preferences.$timeout(angular.bind(_this, _this.pollInbox), refreshViewCheck.timeInterval()*1000);
     });
   };
@@ -865,6 +872,9 @@
     if (preferences.defaults.Forward && preferences.defaults.Forward.forwardAddress)
       preferences.defaults.Forward.forwardAddress = _.compact(preferences.defaults.Forward.forwardAddress);
 
+      if (preferences.defaults.Notification && preferences.defaults.Notification.notificationAddress)
+      preferences.defaults.Notification.notificationAddress = _.compact(preferences.defaults.Notification.notificationAddress);
+
     // Merge back calendar categories colors keys and values
     preferences.defaults.SOGoCalendarCategoriesColors = {};
     _.forEach(preferences.defaults.SOGoCalendarCategories, function(key, i) {
