@@ -129,7 +129,7 @@
 
     // Restart the refresh timer, if needed
     var refreshViewCheck = Component.$Preferences.defaults.SOGoRefreshViewCheck;
-    if (refreshViewCheck && refreshViewCheck != 'manually') {
+    if (refreshViewCheck && refreshViewCheck != 'manually' && !isNaN(refreshViewCheck)) {
       var f = angular.bind(Component.$rootScope, Component.$rootScope.$emit, 'calendars:list');
       Component.$refreshTimeout = Component.$timeout(f, refreshViewCheck.timeInterval()*1000);
     }
@@ -840,8 +840,8 @@
   /**
    * @function hasJitsiUrl
    * @memberof Component.prototype
-   * @desc Check if the there is a jisti url
-   * @returns true if there is a jisti url
+   * @desc Check if the there is a jitsi url
+   * @returns true if there is a jitsi url
    */
   Component.prototype.hasJitsiUrl = function() {
     if (angular.isUndefined(this.attachUrls)) {
